"use strict";
var _Client_metrics, _Client_sdk, _Client_additionalInstrumentations, _Client_isActive;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
const tslib_1 = require("tslib");
const extension_1 = require("./extension");
const config_1 = require("./config");
const metrics_1 = require("./metrics");
const gcProbe = tslib_1.__importStar(require("./probes/v8"));
const internal_logger_1 = require("./internal_logger");
const noops_1 = require("./noops");
const demo_1 = require("./demo");
const version_1 = require("./version");
const helpers_1 = require("./helpers");
const logger_1 = require("./logger");
const api_1 = require("@opentelemetry/api");
const sdk_metrics_1 = require("@opentelemetry/sdk-metrics");
const exporter_metrics_otlp_proto_1 = require("@opentelemetry/exporter-metrics-otlp-proto");
const instrumentation_amqplib_1 = require("@opentelemetry/instrumentation-amqplib");
const opentelemetry_instrumentation_bullmq_1 = require("@appsignal/opentelemetry-instrumentation-bullmq");
const instrumentation_express_1 = require("@opentelemetry/instrumentation-express");
const instrumentation_fastify_1 = require("@opentelemetry/instrumentation-fastify");
const instrumentation_graphql_1 = require("@opentelemetry/instrumentation-graphql");
const instrumentation_http_1 = require("@opentelemetry/instrumentation-http");
const instrumentation_ioredis_1 = require("@opentelemetry/instrumentation-ioredis");
const instrumentation_knex_1 = require("@opentelemetry/instrumentation-knex");
const instrumentation_koa_1 = require("@opentelemetry/instrumentation-koa");
const instrumentation_mongodb_1 = require("@opentelemetry/instrumentation-mongodb");
const instrumentation_mongoose_1 = require("@opentelemetry/instrumentation-mongoose");
const instrumentation_mysql2_1 = require("@opentelemetry/instrumentation-mysql2");
const instrumentation_mysql_1 = require("@opentelemetry/instrumentation-mysql");
const sdk_node_1 = require("@opentelemetry/sdk-node");
const instrumentation_nestjs_core_1 = require("@opentelemetry/instrumentation-nestjs-core");
const instrumentation_pg_1 = require("@opentelemetry/instrumentation-pg");
const instrumentation_1 = require("@prisma/instrumentation");
const serializer_1 = require("./instrumentation/redis/serializer");
const instrumentation_redis_4_1 = require("@opentelemetry/instrumentation-redis-4");
const instrumentation_redis_1 = require("@opentelemetry/instrumentation-redis");
const instrumentation_restify_1 = require("@opentelemetry/instrumentation-restify");
const instrumentation_undici_1 = require("@opentelemetry/instrumentation-undici");
const span_processor_1 = require("./span_processor");
const scheduler_1 = require("./check_in/scheduler");
const DefaultInstrumentations = {
    "@appsignal/opentelemetry-instrumentation-bullmq": opentelemetry_instrumentation_bullmq_1.BullMQInstrumentation,
    "@opentelemetry/instrumentation-amqplib": instrumentation_amqplib_1.AmqplibInstrumentation,
    "@opentelemetry/instrumentation-express": instrumentation_express_1.ExpressInstrumentation,
    "@opentelemetry/instrumentation-fastify": instrumentation_fastify_1.FastifyInstrumentation,
    "@opentelemetry/instrumentation-graphql": instrumentation_graphql_1.GraphQLInstrumentation,
    "@opentelemetry/instrumentation-http": instrumentation_http_1.HttpInstrumentation,
    "@opentelemetry/instrumentation-ioredis": instrumentation_ioredis_1.IORedisInstrumentation,
    "@opentelemetry/instrumentation-knex": instrumentation_knex_1.KnexInstrumentation,
    "@opentelemetry/instrumentation-koa": instrumentation_koa_1.KoaInstrumentation,
    "@opentelemetry/instrumentation-mongodb": instrumentation_mongodb_1.MongoDBInstrumentation,
    "@opentelemetry/instrumentation-mongoose": instrumentation_mongoose_1.MongooseInstrumentation,
    "@opentelemetry/instrumentation-mysql2": instrumentation_mysql2_1.MySQL2Instrumentation,
    "@opentelemetry/instrumentation-mysql": instrumentation_mysql_1.MySQLInstrumentation,
    "@opentelemetry/instrumentation-nestjs-core": instrumentation_nestjs_core_1.NestInstrumentation,
    "@opentelemetry/instrumentation-pg": instrumentation_pg_1.PgInstrumentation,
    "@opentelemetry/instrumentation-redis": instrumentation_redis_1.RedisInstrumentation,
    "@opentelemetry/instrumentation-redis-4": instrumentation_redis_4_1.RedisInstrumentation,
    "@opentelemetry/instrumentation-restify": instrumentation_restify_1.RestifyInstrumentation,
    "@opentelemetry/instrumentation-undici": instrumentation_undici_1.UndiciInstrumentation,
    "@prisma/instrumentation": instrumentation_1.PrismaInstrumentation
};
/**
 * AppSignal for Node.js's main class.
 *
 * Provides methods to control the AppSignal instrumentation and the system
 * agent.
 *
 * @class
 */
class Client {
    /**
     * Global accessors for the AppSignal client
     */
    static get client() {
        return global.__APPSIGNAL__;
    }
    /**
     * Global accessors for the AppSignal Config
     */
    static get config() {
        var _a;
        return (_a = this.client) === null || _a === void 0 ? void 0 : _a.config;
    }
    /**
     * Global accessors for the AppSignal integration Logger
     */
    static get internalLogger() {
        if (this.client) {
            return this.client.internalLogger;
        }
        else {
            return noops_1.noopInternalLogger;
        }
    }
    /**
     * Global accessors for the AppSignal Logger API
     */
    static logger(group, level = "info", format = "plaintext") {
        if (this.client) {
            return this.client.logger(group, level, format);
        }
        else {
            return noops_1.noopLogger;
        }
    }
    /**
     * Starts AppSignal with the given configuration. If no configuration is set
     * yet it will try to automatically load the configuration using the
     * environment loaded from environment variables.
     */
    constructor(options = {}) {
        this.VERSION = version_1.VERSION;
        _Client_metrics.set(this, void 0);
        _Client_sdk.set(this, void 0);
        _Client_additionalInstrumentations.set(this, void 0);
        _Client_isActive.set(this, void 0);
        tslib_1.__classPrivateFieldSet(this, _Client_additionalInstrumentations, options.additionalInstrumentations || [], "f");
        this.config = new config_1.Configuration(options);
        this.extension = new extension_1.Extension();
        this.internalLogger = this.setupInternalLogger();
        this.storeInGlobal();
        // These will be overwritten if AppSignal can be started
        tslib_1.__classPrivateFieldSet(this, _Client_metrics, new noops_1.NoopMetrics(), "f");
        tslib_1.__classPrivateFieldSet(this, _Client_isActive, false, "f");
        if (process.env._APPSIGNAL_DIAGNOSE === "true") {
            this.internalLogger.info("AppSignal not starting: running in diagnose mode");
            return;
        }
        const validationError = this.config.validationError();
        if (validationError) {
            console.info(`AppSignal not starting: ${validationError}`);
            return;
        }
        if (!extension_1.Extension.isLoaded) {
            this.extension.logLoadingErrors();
            console.error("AppSignal not starting: extension failed to load");
            return;
        }
        this.extension.start();
        tslib_1.__classPrivateFieldSet(this, _Client_metrics, new metrics_1.Metrics(), "f");
        tslib_1.__classPrivateFieldSet(this, _Client_isActive, true, "f");
        if (this.config.data.initializeOpentelemetrySdk) {
            tslib_1.__classPrivateFieldSet(this, _Client_sdk, this.initOpenTelemetry(), "f");
            this.setUpOpenTelemetryLogger();
        }
        this.initCoreProbes();
    }
    get isActive() {
        return tslib_1.__classPrivateFieldGet(this, _Client_isActive, "f");
    }
    /**
     * Stops the AppSignal agent.
     *
     * Call this before the end of your program to make sure the
     * agent is stopped as well.
     */
    async stop(calledBy) {
        var _a;
        if (calledBy) {
            console.log(`Stopping AppSignal (${calledBy})`);
        }
        else {
            console.log("Stopping AppSignal");
        }
        this.metrics().probes().stop();
        await Promise.all([(_a = tslib_1.__classPrivateFieldGet(this, _Client_sdk, "f")) === null || _a === void 0 ? void 0 : _a.shutdown(), scheduler_1.scheduler.shutdown()]);
        this.extension.stop();
    }
    /**
     * Internal private function used by the demo CLI.
     *
     * https://docs.appsignal.com/nodejs/3.x/command-line/demo.html
     *
     * @private
     */
    demo() {
        (0, demo_1.demo)(this);
    }
    /**
     * Returns the current `Metrics` object.
     *
     * To track application-wide metrics, you can send custom metrics to AppSignal.
     * These metrics enable you to track anything in your application, from newly
     * registered users to database disk usage. These are not replacements for custom
     * instrumentation, but provide an additional way to make certain data in your
     * code more accessible and measurable over time.
     *
     * With different types of metrics (gauges, counters and measurements)
     * you can track any kind of data from your apps and tag them with metadata
     * to easily spot the differences between contexts.
     */
    metrics() {
        return tslib_1.__classPrivateFieldGet(this, _Client_metrics, "f");
    }
    logger(group, level = "info", format = "plaintext") {
        if (this.isActive) {
            return new logger_1.BaseLogger(this, group, level, format);
        }
        else {
            return noops_1.noopLogger;
        }
    }
    /**
     * Initialises all the available probes to attach automatically at runtime.
     */
    initCoreProbes() {
        const probes = [gcProbe];
        // load probes
        probes.forEach(({ PROBE_NAME, init }) => tslib_1.__classPrivateFieldGet(this, _Client_metrics, "f").probes().register(PROBE_NAME, init(tslib_1.__classPrivateFieldGet(this, _Client_metrics, "f"))));
    }
    defaultInstrumentationsConfig() {
        const sendParams = this.config.data.sendParams;
        const sendSessionData = this.config.data.sendSessionData;
        const requestHeaders = this.config.data.requestHeaders;
        return {
            "@appsignal/opentelemetry-instrumentation-bullmq": {
                emitCreateSpansForBulk: false,
                emitCreateSpansForFlow: true,
                requireParentSpanForPublish: true
            },
            "@opentelemetry/instrumentation-express": {
                requestHook: function (_span, info) {
                    if (info.layerType === instrumentation_express_1.ExpressLayerType.REQUEST_HANDLER) {
                        if (sendParams) {
                            const routeParams = info.request.params;
                            const queryParams = info.request.query;
                            const requestBody = info.request.body;
                            const params = { ...routeParams, ...queryParams, ...requestBody };
                            (0, helpers_1.setParams)(params);
                        }
                        if (sendSessionData) {
                            (0, helpers_1.setSessionData)(info.request.cookies);
                        }
                    }
                }
            },
            "@opentelemetry/instrumentation-fastify": {
                requestHook: function (_span, info) {
                    const queryParams = info.request.query;
                    const requestBody = info.request.body;
                    const params = { ...queryParams, ...requestBody };
                    (0, helpers_1.setParams)(params);
                }
            },
            "@opentelemetry/instrumentation-graphql": {
                ignoreTrivialResolveSpans: true,
                mergeItems: true,
                allowValues: false
            },
            "@opentelemetry/instrumentation-http": {
                headersToSpanAttributes: {
                    server: { requestHeaders }
                }
            },
            "@opentelemetry/instrumentation-ioredis": {
                dbStatementSerializer: serializer_1.RedisDbStatementSerializer
            },
            "@opentelemetry/instrumentation-koa": {
                requestHook: function (span, info) {
                    if (sendParams && info.layerType === instrumentation_koa_1.KoaLayerType.ROUTER) {
                        const queryParams = info.context.request.query;
                        (0, helpers_1.setParams)(queryParams, span);
                    }
                }
            },
            "@opentelemetry/instrumentation-redis": {
                dbStatementSerializer: serializer_1.RedisDbStatementSerializer
            },
            "@opentelemetry/instrumentation-redis-4": {
                dbStatementSerializer: serializer_1.RedisDbStatementSerializer
            },
            "@opentelemetry/instrumentation-restify": {
                requestHook: (span, info) => {
                    if (sendParams &&
                        info.layerType === instrumentation_restify_1.LayerType.REQUEST_HANDLER) {
                        const request = info.request;
                        const params = Object.assign(request.params || {}, request.query || {});
                        (0, helpers_1.setParams)(params, span);
                    }
                }
            },
            "@opentelemetry/instrumentation-undici": {
                requireParentforSpans: true
            },
            "@prisma/instrumentation": {
                middleware: true
            }
        };
    }
    defaultInstrumentations() {
        const disabledInstrumentations = this.config.data.disableDefaultInstrumentations;
        if (disabledInstrumentations === true) {
            return [];
        }
        const instrumentationConfigs = this.defaultInstrumentationsConfig();
        return Object.entries(DefaultInstrumentations)
            .filter(([name, _constructor]) => !(disabledInstrumentations || []).includes(name))
            .map(([name, constructor]) => new constructor(instrumentationConfigs[name] || {}));
    }
    opentelemetryInstrumentations() {
        return tslib_1.__classPrivateFieldGet(this, _Client_additionalInstrumentations, "f").concat(this.defaultInstrumentations());
    }
    /**
     * Initialises OpenTelemetry instrumentation
     */
    initOpenTelemetry() {
        const testMode = process.env["_APPSIGNAL_TEST_MODE"];
        const testModeFilePath = process.env["_APPSIGNAL_TEST_MODE_FILE_PATH"];
        let spanProcessor;
        if (testMode && testModeFilePath) {
            spanProcessor = new span_processor_1.TestModeSpanProcessor(testModeFilePath);
        }
        else {
            spanProcessor = new span_processor_1.SpanProcessor(this);
        }
        const instrumentations = this.opentelemetryInstrumentations();
        // Metrics initialization
        let metricReader;
        if (this.config.data["enableOpentelemetryHttp"]) {
            metricReader = new sdk_metrics_1.PeriodicExportingMetricReader({
                exporter: new PatchedOTLPMetricExporter({
                    url: `http://localhost:${this.config.data["opentelemetryPort"]}/v1/metrics`
                })
            });
        }
        const sdk = new sdk_node_1.NodeSDK({
            instrumentations,
            spanProcessor,
            metricReader
        });
        sdk.start();
        return sdk;
    }
    /**
     * Sets up the AppSignal logger with the output based on the `log` config option. If
     * the log file is not accessible, stdout will be the output.
     */
    setupInternalLogger() {
        const logFilePath = this.config.logFilePath;
        const logLevel = String(this.config.data["logLevel"]);
        const logType = String(this.config.data["log"]);
        let logger;
        if (logType == "file" && logFilePath) {
            logger = new internal_logger_1.BaseInternalLogger(logType, logLevel, logFilePath);
        }
        else {
            logger = new internal_logger_1.BaseInternalLogger(logType, logLevel);
        }
        return logger;
    }
    /**
     * Sets up the OpenTelemetry diag logger based on our integration logger level.
     * If our integration logger level is "silly" ("trace"), the OpenTelemetry diag debug messages
     * are logged.
     */
    setUpOpenTelemetryLogger() {
        if (this.config.data["logLevel"] === "trace") {
            api_1.diag.setLogger(new api_1.DiagConsoleLogger(), api_1.DiagLogLevel.DEBUG);
        }
    }
    /**
     * Stores the client in global object after initializing
     */
    storeInGlobal() {
        global.__APPSIGNAL__ = this;
    }
    get tracer() {
        console.error("The `appsignal.tracer()` function was called, but it has been removed in AppSignal for Node.js package version 3.x. Please read our migration guide to upgrade to this new version of our package: https://docs.appsignal.com/nodejs/3.x/migration-guide.html. It is also possible to downgrade to version 2.x, after which this code will work again.");
        return () => { };
    }
}
exports.Client = Client;
_Client_metrics = new WeakMap(), _Client_sdk = new WeakMap(), _Client_additionalInstrumentations = new WeakMap(), _Client_isActive = new WeakMap();
// Custom metrics aggregation temporality selector
// This requires a patch because the Node.js exporter doesn't allow this to be
// specified per metric type.
const customAggregationTemporality = {
    COUNTER: sdk_metrics_1.AggregationTemporality.DELTA,
    UP_DOWN_COUNTER: sdk_metrics_1.AggregationTemporality.DELTA,
    OBSERVABLE_COUNTER: sdk_metrics_1.AggregationTemporality.DELTA,
    OBSERVABLE_GAUGE: sdk_metrics_1.AggregationTemporality.CUMULATIVE,
    OBSERVABLE_UP_DOWN_COUNTER: sdk_metrics_1.AggregationTemporality.DELTA,
    HISTOGRAM: sdk_metrics_1.AggregationTemporality.DELTA
};
class PatchedOTLPMetricExporter extends exporter_metrics_otlp_proto_1.OTLPMetricExporter {
    selectAggregationTemporality(instrumentType) {
        const aggregationTemporality = customAggregationTemporality[instrumentType];
        return aggregationTemporality != undefined
            ? aggregationTemporality
            : super.selectAggregationTemporality(instrumentType);
    }
}
