"use strict";
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const helpers_1 = require("../scripts/extension/support/helpers");
let mod;
function fetchInstalledArch() {
    try {
        const rawReport = fs_1.default.readFileSync((0, helpers_1.reportPath)(), "utf8");
        const buildReport = JSON.parse(rawReport)["build"];
        return [buildReport["architecture"], buildReport["target"]];
    }
    catch (error) {
        console.error("[appsignal][ERROR] Unable to fetch install report:", error);
        return [];
    }
}
try {
    mod = require("../build/Release/extension.node");
    mod.isLoaded = true;
}
catch (error) {
    mod = {
        isLoaded: false,
        extension: {
            start() {
                return;
            },
            stop() {
                return;
            },
            diagnoseRaw() {
                return;
            },
            runningInContainer() {
                return;
            },
            createOpenTelemetrySpan() {
                return;
            },
            log() {
                return;
            },
            logLoadingErrors() {
                const [installArch, installTarget] = fetchInstalledArch();
                const arch = process.arch;
                const target = (0, helpers_1.processTarget)();
                if (installArch &&
                    installTarget &&
                    (arch !== installArch || target !== installTarget)) {
                    console.error(`[appsignal][ERROR] The AppSignal extension was installed for architecture '${installArch}-${installTarget}', but the current architecture is '${arch}-${target}'. Please reinstall the AppSignal package on the host the app is started.`);
                }
                else {
                    console.error("[appsignal][ERROR] AppSignal failed to load the extension. Please run the diagnose tool and email us at support@appsignal.com: https://docs.appsignal.com/nodejs/3.x/command-line/diagnose.html\n", error);
                }
            }
        }
    };
}
module.exports = mod;
