"use strict";
const tslib_1 = require("tslib");
const logger_1 = require("./logger");
const pino_abstract_transport_1 = tslib_1.__importDefault(require("pino-abstract-transport"));
const extension_1 = require("./extension");
const appsignalPinoTransport = ({ group = "app" }) => {
    return (0, pino_abstract_transport_1.default)(async (source) => {
        // We expect this code to be running in a worker thread and for the
        // extension to have already been loaded and started on the main thread.
        // Since we expect the extension to already have been initialised, we
        // pass `{ active: false }` to avoid initialising it again.
        const extension = new extension_1.Extension({ active: false });
        for await (const obj of source) {
            sendLogs(extension, group, parseInfo(obj));
        }
    });
};
async function sendLogs(extension, group, data) {
    extension.log(group, data.severity, logger_1.LOGGER_FORMAT.plaintext, data.message, data.attributes);
}
function parseInfo(obj) {
    const { level, msg, ...attributes } = obj;
    return {
        severity: getSeverity(level),
        message: msg,
        attributes: flattenAttributes(attributes)
    };
}
function flattenAttributes(attributes, prefix = "") {
    let result = {};
    for (const key in attributes) {
        const newKey = prefix ? `${prefix}.${key}` : key;
        if (typeof attributes[key] === "object" &&
            attributes[key] !== null &&
            !Array.isArray(attributes[key])) {
            const flattened = flattenAttributes(attributes[key], newKey);
            result = { ...result, ...flattened };
        }
        else {
            result[newKey] = attributes[key];
        }
    }
    return result;
}
function getSeverity(level) {
    if (level >= 50)
        return logger_1.LOGGER_LEVEL_SEVERITY.error;
    if (level >= 40)
        return logger_1.LOGGER_LEVEL_SEVERITY.warn;
    if (level >= 30)
        return logger_1.LOGGER_LEVEL_SEVERITY.info;
    if (level >= 20)
        return logger_1.LOGGER_LEVEL_SEVERITY.debug;
    return logger_1.LOGGER_LEVEL_SEVERITY.trace;
}
module.exports = appsignalPinoTransport;
